#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Automatic verification of the best available pairwise distance function. 
By default is deactivated by flag *bypassCheck* and the used function defaults to 
*euclidean_distances* from scikit-learn. 

@author: Joakim Linja
"""

import numpy as np
from time import process_time, time
from sklearn.metrics import euclidean_distances
from sklearn.metrics import pairwise_distances
from scipy.spatial.distance import cdist

bypassCheck = True

funs = [cdist, euclidean_distances, pairwise_distances]
strs = ['cdist','euclidean_distances','pairwise_distances']

if bypassCheck:
    print('Bypassing pairwise distance function check')
    s2 = 2
    s1 = 1
    print('Selected '+strs[s2])

else: 
    _pdc_a = np.random.rand(5000,4000)
    _pdc_b = np.random.rand(2000,4000)
    
    _pdc_t1 = process_time()
    _pdc_w1 = time()
    _pdc_cd = cdist(_pdc_a,_pdc_b)
    _pdc_t2 = process_time()
    _pdc_w2 = time()
    _pdc_ed = euclidean_distances(_pdc_a,_pdc_b)
    _pdc_t3 = process_time()
    _pdc_w3 = time()
    _pdc_pw = pairwise_distances(_pdc_a,_pdc_b)
    _pdc_t4 = process_time()
    _pdc_w4 = time()
    
    _pdc_rt_cd = _pdc_t2-_pdc_t1
    _pdc_rt_ed = _pdc_t3-_pdc_t2
    _pdc_rt_pw = _pdc_t4-_pdc_t3
    _pdc_rw_cd = _pdc_w2-_pdc_w1
    _pdc_rw_ed = _pdc_w3-_pdc_w2
    _pdc_rw_pw = _pdc_w4-_pdc_w3
    
    
    
    rts = np.array([_pdc_rt_cd,_pdc_rt_ed,_pdc_rt_pw])
    rws = np.array([_pdc_rw_cd,_pdc_rw_ed,_pdc_rw_pw])
    s1 = np.argsort(rts)[0]
    s2 = np.argsort(rws)[0]
    
    print('Pairwise distance check')
    print('Selected '+strs[s2])
    if s1!=s2:
        print('Fastest process time was '+strs[s1])
    print('pdcs:',strs)
    print('proc:',rts)
    print('wall:',rws)
    print('')
    del _pdc_a
    del _pdc_b
    del _pdc_t1
    del _pdc_w1
    del _pdc_cd
    del _pdc_t2
    del _pdc_w2
    del _pdc_ed
    del _pdc_t3
    del _pdc_w3
    del _pdc_pw
    del _pdc_t4
    del _pdc_w4
    del _pdc_rt_cd
    del _pdc_rt_ed
    del _pdc_rt_pw
    del _pdc_rw_cd
    del _pdc_rw_ed
    del _pdc_rw_pw
    del rts
    del rws


_pdcfun = funs[s2]

del s1
del s2
del funs
del strs


def pdc(*args):
    """
    Parameters
    ----------
    *args : 
        Arguments to be given to the pairwise distance calculation function

    Returns
    -------
    Distance matrix
        Calculated distances for the given input. 
    """
    
    return _pdcfun(*args)
